/*
 ============================================================================
 Name		 : ActiveObject.cpp
 Copyright   : Symbian Academy, 2007
 Description : Exercise: Active Objects
 ============================================================================
 */

//  Include Files  

#include <e32base.h>
#include <e32std.h>
#include <e32cons.h>            // Console

#include "Engine.h"
#include "UpdateTimer.h"
#include "ConsoleKeys.h"

// Function prototype (C-style).
LOCAL_C void MainL();
GLDEF_C TInt E32Main();

//  Global Variables
LOCAL_D CConsoleBase* console;

LOCAL_C void MainL()
	{
	// ------------------------------------------------------------------
	// Initialization
	// ------------------------------------------------------------------
	// Hide the cursor
	console->SetCursorHeight(0);

	// Create the game engine
	CEngine* engine = CEngine::NewLC(console);

	// Assign the settings of the game:
	// For S60 emulator in QVGA mode (Default for S60 3rd Ed., FP1):
	//engine->SetParametersL(50, 24, 27, 15, 15, 2);
	// For S60 emulator in Double size mode (Default for S60 3rd Ed., MR):
	engine->SetParametersL(50, 32, 40, 18, 15, 2);

	// Edit 32: Create the key listener active object (CConsoleKeys) through its
	//          NewLC()-method. It requires the console and the engine as parameters.
	

	// Edit 18: Create the update timer active object (CUpdateTimer) using
	//          the NewLC()-method. It requires the engine as a parameter
	//          to be able to update the world.
	

	// Edit 19: Give the engine a reference to the update timer, so that the engine
	//          can increase the speed of the game over time.
	//          Uncomment the following line.
	//engine->SetUpdateTimer(updateTimer);

	// Edit 20: Issue the first request to the update timer to start the process.
	//          This is done through the StartProcess function. A good initial delay would
	//          be 40000.
	//          Important: This has to be done before starting the active scheduler!
	
	
	// Edit 33: Issue the first request to the key listener.
	

	// Edit 21: Start the active scheduler.
	//          This function completes when the scheduler stops. In the meantime,
	//          it will wait for callbacks e.g. from asynchronous service providers.
	

	// Edit 22, 34: Increase the number of objects to destroy through
	//              the cleanup stack so that it matches the number of objects
	//              you have created with NewLC() in this function.
	//              Remove from the cleanup stack and destroy:
	//              1. the updateTimer
	//              2. the consolekeys
	//              3. the engine
	CleanupStack::PopAndDestroy(1);
	}

LOCAL_C void DoStartL()
	{
	// Create active scheduler (to run active objects)
	CActiveScheduler* scheduler = new (ELeave) CActiveScheduler();

	// Push onto the cleanup stack
	CleanupStack::PushL(scheduler);

	// Install as the active scheduler
	CActiveScheduler::Install(scheduler);

	MainL();

	// Delete active scheduler
	CleanupStack::PopAndDestroy(scheduler);
	}

// ------------------------------------------------------------------
// Main Application
// ------------------------------------------------------------------
GLDEF_C TInt E32Main()
	{
	// Create cleanup stack
	CTrapCleanup* cleanup = CTrapCleanup::New ();

	// Create output console
	_LIT(KTextConsoleTitle, "Console - Active Objects");
	TRAPD(createError, console = Console::NewL(KTextConsoleTitle, TSize(KConsFullScreen,KConsFullScreen)))
	;
	if ( createError)
		return createError;

	// Run application code inside TRAP harness
	__UHEAP_MARK;

	TRAPD(mainError, DoStartL())
	;
	_LIT(KTextFailed, "DoStartL() failed, leave code = %d\n");
	if ( mainError)
		{
		console->Printf (KTextFailed, mainError);
		}

	// Wait for the space key before exiting.
	// Generally better than accepting any key to prevent immediate shutdown
	// if a key has still been in the queue when the app. started.
	_LIT(KTextPressSpace, "\n[Press space to exit]\n");
	console->Printf (KTextPressSpace);
	while (console->Getch ()!= EKeySpace)
		{
		}

	// Check the state of the heap memory - if the same amount of memory
	// is available as before MainL() was executed, everything's fine.
	// Otherwise, a panic will be raised, which would result in a graphical
	// error dialog in an UI framework, but exits the application when
	// using the console mode.
	__UHEAP_MARKEND;

	// If the program passed the __UHEAP_MARKEND-macro, no memory
	// leaks were detected
	_LIT(KTextMemoryLeak, "No memory leaks detected! [Press space]\n");
	console->Printf (KTextMemoryLeak);
	while (console->Getch ()!= EKeySpace)
		{
		}

	// Cleanup
	delete console;
	delete cleanup;
	return KErrNone;
	}
